package com.jware.apps.scheduler.web.struts.schedule;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.util.MessageResources;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SimpleTrigger;
import org.quartz.Trigger;

import com.jware.apps.scheduler.util.JobSchedulerUtils;
import com.jware.base.web.struts.jWareBaseAction;

/**
 * @author Jrmt
*/
public final class JobTriggerEdit extends jWareBaseAction {

    /**
     * Commons Logging instance.
    */
    private Log log = LogFactory.getLog(this.getClass().getName());

	/** 
	 * Process the specified HTTP request, and create the corresponding HTTP
	 * response (or forward to another web component that will create it).
	 * Return an <code>ActionForward</code> instance describing where and how
	 * control should be forwarded, or <code>null</code> if the response has
	 * already been completed.
	 *
	 * @param mapping The ActionMapping used to select this instance
	 * @param actionForm The optional ActionForm bean for this request (if any)
	 * @param request The HTTP request we are processing
	 * @param response The HTTP response we are creating
	 * @param errors The ActionErrors created in BaseAction
	 * @param mochFlag The flag using Mocha
	 *
	 * @exception Exception if an input/output error or servlet exception occurs
	 */
	public ActionForward doExecute(ActionMapping mapping,
				 ActionForm form,
				 HttpServletRequest request,
				 HttpServletResponse response,
				 ActionErrors errors,
				 boolean mochaFlag)
	throws Exception {

		// Extract attributes we will need
		HttpSession session = request.getSession();
		Locale locale = getLocale(request);
		
		JobTriggerForm info = (JobTriggerForm)form;
		
		MessageResources messages = getResources(request);
		
		if (log.isDebugEnabled()) {
			log.debug("JobTriggerEdit:  Processing actions");
		}

		if( info == null ){
			info = new JobTriggerForm();
		}
		JobDetail jobDetail = null;
		String triggerName = request.getParameter( "triggerName" );
		String groupName = request.getParameter( "groupName" );
		if( triggerName != null && groupName != null ){

			Scheduler scheduler = JobSchedulerUtils.getCurrentScheduler(session);

			Trigger trigger = scheduler.getTrigger(triggerName, groupName);
			if( trigger != null ){
				
				info.setJobName(trigger.getJobName());
				info.setJobGroup( trigger.getJobGroup() );
				info.setTriggerName( trigger.getName() );
				info.setTriggerGroup( trigger.getGroup() );
				info.setStartTime( JobSchedulerUtils.getDateAsString(trigger.getStartTime()) );
				info.setStopTime( JobSchedulerUtils.getDateAsString(trigger.getEndTime()) );
				info.setVolatility( trigger.isVolatile() );
				info.setMisFireInstruction( trigger.getMisfireInstruction() );
				info.setDescription( trigger.getDescription( ) );
				info.setNextFireTime( JobSchedulerUtils.getDateAsString(trigger.getNextFireTime()) );
				info.setPreviousFireTime( JobSchedulerUtils.getDateAsString(trigger.getPreviousFireTime()) );
				if( trigger instanceof SimpleTrigger ){ 
					info.setType( "simple" );
				} else {
					info.setType( "cron" );
				}
				
				request.setAttribute( "JobTriggerForm", info );
				
				return mapping.findForward( info.getType() );
				
			}
		}
		
		return mapping.findForward("failure");
		
    }

	
}

